api = (function () {
    'use strict';
 
    var evt = model.events,
        apiCallback = {},
        config, 
        player, 
        errorMsg,
        isReady,
        isInitiated;
        
    doc.addEventListener(evt.AD_PLUGIN_ERROR, function (e) {
        errorMsg = e.detail.message;
    });
    doc.addEventListener(evt.AD_PLUGIN_READY, function (e) {
        isReady = true;
    });

    /* ==========================================================================
        PUBLIC
    ========================================================================== */
    return {
        init : function (elem, conf) {
            if (isInitiated) {
                return this;
            }
            isInitiated = true;
            player = elem;
            config = conf;
            doc.dispatchEvent(new CustomEvent(model.events.AD_PLUGIN_INIT));
            return this; 
        },
        utils : function () {
            return utils.helpers;
        },
        getConfig : function () {
            return config;
        },
        getVideoElem : function () {
            return player;
        },
        trigger : function (evt, param) {
            if (apiCallback[evt]) {
                apiCallback[evt](param);
            }
        },
        onError : function (callback) {
            apiCallback.onError = callback;
            if(!isReady) {
                callback(errorMsg);
            }
            return this;
        },
        onReady : function (callback) {
            apiCallback.onReady = callback;
            if(isReady) {
                callback();
            }
            return this;
        },
        onAdStart : function (callback) {
            apiCallback.onAdStart = callback;
            return this;
        },
        onAdCompleted : function (callback) {
            apiCallback.onAdCompleted = callback;
            return this;
        },
        onAdSlotStart : function (callback) {
            apiCallback.onAdSlotStart = callback;
            return this;
        },
        onAdSlotCompleted : function (callback) {
            apiCallback.onAdSlotCompleted = callback;
            return this;
        },
        onAdClickThrough : function (callback) {
            apiCallback.onAdClickThrough = callback;
            return this;
        },
        onAdSkipButtonClick : function(callback) {
            apiCallback.onContentCompleted = callback;
            return this;
        },
        onContentStart : function (callback) {
            apiCallback.onContentStart = callback;
            return this;
        },
        onContentCompleted : function (callback) {
            apiCallback.onContentCompleted = callback;
            return this;
        }
    };
}());