core.adRequest = (function () {
    'use strict';

	var MAX_BIT_RATE = 800, //bitrate is capped to 800, change to increase quality (and size)
		l = utils.logger,
		evt = model.events,
		adRequesterSettings,
		adSession,
		adRequester;

	function configAdRequest (config) {
		adRequesterSettings = {
			deviceContainer : (config.devicecontainer) ? config.devicecontainer : '',
			persistentId : (config.persistentid) ? config.persistentid : ''
		};

		adRequester = new videoplaza.adrequest.AdRequester(config.host, adRequesterSettings);

		adRequester.addLogListener(function(logObj){
			l.error('adRequest', logObj.message);
		});
	}

	function onAdRequestFail (msg) {
		l.error('adRequest', msg);
		doc.dispatchEvent(new CustomEvent(evt.AD_REQUEST_FAIL));
	}

	function onAdRequestSuccess (session) {
		l.log('adRequest', 'ad request success');
		l.info('adRequest', 'ad session: ');
		l.log(session);

		adSession = session;

		doc.dispatchEvent(new CustomEvent(evt.AD_REQUEST_SUCCESS));
	}

	/* ==========================================================================
		Public
	========================================================================== */
	return {
		newAdRequest : function (config, duration) {
			l.log('adRequest', 'start new ad request');

			configAdRequest(config);

			config.duration = duration || 0;

			//exclude unsupported formats
			if (config.flags) {
				config.flags.push('noskins','nooverlays');
			}
			else {
				config.flags = ['noskins','nooverlays'];
			}

			var cues = config.cuepoints || [];
			if (typeof cues ==='string') {
				config.cuepoints = cues.split(",").map(Number);
			}

			var contentMetadata = config,
				requestSettings = {
					height : config.height || 0,
					width : config.width || 0,
					maxBitRate : config.maxbitrate || MAX_BIT_RATE,
					referrerUrl : config.referrerurl || '',
					//linearPlaybackPositions : cues,
					insertionPointTypes : [
						//enable if asking for midroll ads
						//videoplaza.adrequest.AdRequester.InsertionPointType.PLAYBACK_POSITION,
						videoplaza.adrequest.AdRequester.InsertionPointType.ON_BEFORE_CONTENT,
						videoplaza.adrequest.AdRequester.InsertionPointType.ON_CONTENT_END
					]
				};

			l.info('adRequest', 'contentMetadata: ');
			l.info(contentMetadata);
			l.info('adRequest', 'requestSettings: ');
			l.info(requestSettings);

			adRequester.requestSession(contentMetadata, requestSettings, onAdRequestSuccess, onAdRequestFail);
		},
		loadThirdPartyAd : function  (ad) {
			adRequester.requestThirdParty(ad, function(loadedAd) {
				if(loadedAd.ready){
					l.log('adRequest', 'Third party ad loaded');
					doc.dispatchEvent(new CustomEvent(evt.AD_REQUEST_THIRD_PARTY_SUCCESS));
				} else {
					l.error('adRequest', 'Third party ad fail to load');
					doc.dispatchEvent(new CustomEvent(evt.AD_REQUEST_THIRD_PARTY_FAIL));
				}
			});
		},
		getAdSession: function () {
			return adSession;
		}
	};
}());