core.adTracker = (function () {
    'use strict';

    var l = utils.logger,
        session, ad, firstQuartile, midpoint, thirdQuartile,
        duration, creative, tracker;

    function trackEvent (elm, evt) {
        l.event('adTracker','track event: ' + evt, '#FF9933');

        if (!elm || !tracker) {
            l.error('adTracker','failt to track event: ' + evt);
            return;
        }
        tracker.trackEvent(elm, evt);
    }

    function reportError (elm, evt) {
        l.event('adTracker','track error: ' + evt, '#FF9933');

        if (!elm || !tracker) {
            l.error('adTracker','fail to report error event: ' + evt);
            return;
        }
        tracker.reportError(elm, evt);
    }

    /* ==========================================================================
    	Public
    ========================================================================== */
    return {
        init : function () {
            tracker = new videoplaza.tracking.Tracker();
            tracker.addLogListener(function(logObj){
                l.error('adTracker', logObj.message);
            });
        },
        setTrackingElements : function (adSession, currentAd) {
            session = adSession;
            ad = currentAd;

            //inventory ads have no duration
            if (!ad.creatives[0]) {
                return;
            }

            creative = ad.creatives[0];
            duration = creative.duration; 
            firstQuartile = Math.round(duration * 0.25);
            midpoint = Math.round(duration * 0.50);
            thirdQuartile = Math.round(duration * 0.75);
        },
        quartile : function (evt) {
            switch (Math.round(evt.target.currentTime)) {
                case firstQuartile:
                    trackEvent(creative, videoplaza.tracking.Tracker.CreativeEventType.FIRST_QUARTILE);
                    firstQuartile = NaN; //stop tracking it twice
                    return;
                case midpoint:
                    trackEvent(creative, videoplaza.tracking.Tracker.CreativeEventType.MIDPOINT);
                    midpoint = NaN;
                    return;
                case thirdQuartile:
                    trackEvent(creative, videoplaza.tracking.Tracker.CreativeEventType.THIRD_QUARTILE);
                    thirdQuartile = NaN;
                    return;
            }
        },
        impression : function () {
            trackEvent(ad, videoplaza.tracking.Tracker.AdEventType.IMPRESSION);
        },
        start : function () {
            trackEvent(creative, videoplaza.tracking.Tracker.CreativeEventType.START);
        },
        complete : function () {
            trackEvent(creative, videoplaza.tracking.Tracker.CreativeEventType.COMPLETE);
        },
        close : function () {
            trackEvent(creative, videoplaza.tracking.Tracker.CreativeEventType.CLOSE);
        },
        content_start : function () {
            trackEvent(session, videoplaza.tracking.Tracker.SessionEventType.CONTENT_START);
        },
        clickthrough : function () {
            trackEvent(creative, videoplaza.tracking.Tracker.CreativeEventType.CLICKTHROUGH);
        },
        creative_view : function (companion) {
            trackEvent(companion, videoplaza.tracking.Tracker.CreativeEventType.CREATIVE_VIEW);
        },

        // error events

        no_ad : function () {
            reportError(ad, videoplaza.tracking.Tracker.AdError.NO_AD);
        },
        type_not_supported : function () {
            reportError(ad, videoplaza.tracking.Tracker.AdError.TYPE_NOT_SUPPORTED);
        },
        general_error : function () {
            reportError(ad, videoplaza.tracking.Tracker.AdError.GENERAL_ERROR);
        },
        media_file_not_found : function () {
           reportError(creative, videoplaza.tracking.Tracker.CreativeError.MEDIA_FILE_NOT_FOUND);
        },
        no_supported_media_file_found : function () {
            reportError(creative, videoplaza.tracking.Tracker.CreativeError.NO_SUPPORTED_MEDIA_FILE_FOUND);
        },
        media_file_display_error : function () {
           reportError(creative, videoplaza.tracking.Tracker.CreativeError.MEDIA_FILE_DISPLAY_ERROR);
        }
    };
}());