/**
 * @author Nelson Diotto
 * @date 2015.03.20
 */
main = (function () {
    'use strict';
    
    var l = utils.logger,
    	help = utils.helpers,
		evt = model.events,
		adRequest = core.adRequest,
		adResponse = core.adResponse,
		adTracker = core.adTracker,
		topMenu = view.topMenu,
		companion = view.companionBanner,
		adsArr = [],
		adSlotArr = [],
		pluginInterface,
		player,
		config,
		playerContentSrc,
		playerPoster,
		adSession,
		clickThroughUrl,
		adType,
		adCreative;

	l.info('main','load HTML5 SDK: ' + videoplaza.versionNumber + ' release: ' + videoplaza.buildDate);

	function displayPreroll() {
		l.log('main', 'start preroll');
		selectAdType(videoplaza.adrequest.AdRequester.InsertionPointType.ON_BEFORE_CONTENT);
	}

	function displayPostroll() {
		l.log('main', 'start postroll');
		selectAdType(videoplaza.adrequest.AdRequester.InsertionPointType.ON_CONTENT_END);
	}

	/*
		2. get specific ad type (pre, mid or post-roll) from the ad response
	 */
	function selectAdType (type) {
		adType = type;
		adsArr = adResponse.getAds(adSession, adType);
		adSlotArr = adsArr.slice(); //clone array
		checkAvailableAds(true);
	}

	/*
		3. check if there available ads to play in the ad response
	 */
	function checkAvailableAds (isFirstAd) {
		if (!isFirstAd) {
			adsArr.shift(); //remove previous ad from array
		}

		l.info('main', 'available ads: ' + adsArr.length + ' type: ' + adType);

		if(adsArr.length > 0 ){
			checkAdReady();
		} else {
			onAdSlotCompleted();
		}
	}

	/*
		4. check if ad is ready
	 */
	function checkAdReady () {
		var isReady = adsArr[0].ready;
		l.log('main', 'ad is ready? ' + isReady);

		if(isReady) {
			ckeckAdType();
		} else {
			adRequest.loadThirdPartyAd(adsArr[0]);
		}
	}

	/*
		5. check if the ad is the correct type
	 */
	function ckeckAdType () {
		l.log('main', 'ad type: ' + adsArr[0].type);

		adTracker.setTrackingElements(adSession, adsArr[0]);

		if (adsArr[0].type === 'inventory') {
			adTracker.no_ad();
			checkAvailableAds();
		}
		else {
			startAd();
		}
	}

	/*
		6. If ad is the correct type, start ad playback 
	 */
	function startAd () {
		l.log('main', 'ad id: ' + adsArr[0].id);

		adCreative = adsArr[0].creatives[0];
		var videoAd = help.findSupportedVideo(player, adCreative.mediaFiles);

		if (videoAd) {
			topMenu.build(player, adsArr[0], config.width, config.height);
			playVideoAsset(videoAd, onAdStart);
			onAdSlotStart();
		}
		else {
			l.error('main', 'not supported media file found');
			adTracker.no_supported_media_file_found();
			checkAvailableAds();
		}
	}

	function startContent() {
		playVideoAsset(playerContentSrc, onContentStart);
	}

	function resetPlayer () {
		l.log('main', 'reset player ' + playerContentSrc +" | "+ playerPoster);
		player.src = playerContentSrc;
		player.poster = playerPoster;

		//stop player restarting after postroll if autoplay
		if (player.autoplay) {
			player.addEventListener('loadstart', function onLoadstart() {
				this.removeEventListener('loadstart', onLoadstart);
				player.pause();
			});
		}

		// comment below to stop showing ads on re-play
		player.addEventListener('play', onPlay);
	}

	function playVideoAsset (src, callback) {
		l.log('main', 'play video: ' + src);

		if (!src) {
			l.error('main', 'undefined video src');
			return;
		}
		player.src = src;

		player.addEventListener('error', onAdStartError);
		player.addEventListener('loadstart', function onLoadstart() {
			this.removeEventListener('loadstart', onLoadstart);

			player.addEventListener('canplaythrough', callback);
			player.poster = '';
			player.play();
		});
	}

	function enableClickThrough (enable) {
		if (!enable) {
			player.removeEventListener(clickEvent, onAdClickThrough);
			return;
		}

		clickThroughUrl = adCreative.clickThroughUrl;
		if (clickThroughUrl !== '') {
			player.addEventListener(clickEvent, onAdClickThrough);
		}
		l.log('main', 'enable  clickThrough: ' + enable + ' url: ' + clickThroughUrl);
	}
	/* ==========================================================================
		Callbacks
	========================================================================== */
	/*
		1. as soon the content start playing, pause and make a new
		ad request
	 */
	function onPlay() {
		l.event('main', 'onPlay');
		player.removeEventListener('play', onPlay);
		player.pause();

		//fix for IE returning currentSrc null
		if (help.isIE()) {
			playerContentSrc = player.getAttribute("src") || player.children[0].getAttribute("src");
		}
		else {
			playerContentSrc = player.currentSrc;
		}

		playerPoster = player.poster;
		adRequest.newAdRequest(config, player.duration, player.height, player.width);
	}

	function onAdPluginInit () {
		l.event('main', 'onAdPluginInit');
		config = help.objectToLowerCase(api.getConfig());
		player = api.getVideoElem();

		//FAIL: client host is missing
        if(!help.inObject(config, 'host')) {
        	onAdPluginError('invalid plugin config Object');
            return;
        }
        //FAIL: <video> element is not correct
        else if(!help.isVideo(player)) {
        	onAdPluginError('invalid <video> DOM element');
            return;
        }
        //SUCCESS: start plugin
        onAdPluginReady();
	}

	function onAdPluginError(message) {
		l.error('main', 'onAdPluginError');
        doc.dispatchEvent(new CustomEvent(evt.AD_PLUGIN_ERROR, { 
            'detail': {
                'message' : message
            }
        }));
	}

	function onAdPluginReady () {
		l.event('main', 'onAdPluginReady');
        doc.dispatchEvent(new CustomEvent(evt.AD_PLUGIN_READY));
		player.addEventListener('play', onPlay);
	}

	function onAdRequestFail () {
		l.error('main', 'onAdRequestFail');
		api.trigger('onError','Ad Request Fail');
		resetPlayer();
		startContent();
	}

	function onAdRequestSuccess () {
		l.event('main', 'onAdRequestSuccess');
		adSession = adRequest.getAdSession();
		adTracker.init();

		if (adSession) {
			displayPreroll();
		}
		else {
			l.error('main', 'session undefined');
		}
	}

	function onAdStart () {
		l.event('main', 'onAdStart');
		player.removeEventListener('canplaythrough', onAdStart);
		player.addEventListener('ended', onAdCompleted);
		player.addEventListener('timeupdate', onAdTimeUpdate);
		doc.addEventListener(evt.AD_SKIP_BUTTON_CLICK, onAdSkipButtonClick);
		doc.addEventListener(evt.AD_COMPANION_VIEW, onAdCompanionView);

		enableClickThrough(true);
		topMenu.show(true);
		companion.show(adsArr[0].companions);
		adTracker.impression(adsArr[0]);
		adTracker.start();
		api.trigger('onAdStart',adsArr[0]);
	}

	function onAdStartError (evt) {
		// fix for FF firing error even when media is working
		if (!evt.target.error) {
			return;
		}

		l.error('main', 'onAdStartError ' + evt.target.error.code);
		api.trigger('onError','Fail to Start Video Ad');
		player.removeEventListener('error', onAdStartError);
		adTracker.media_file_display_error();
		player.pause();
		checkAvailableAds();
	}

	function onAdCompleted () {
		l.event('main', 'onAdCompleted');
		player.removeEventListener('ended', onAdCompleted);
		player.removeEventListener('timeupdate', onAdTimeUpdate);
		enableClickThrough(false);
		topMenu.show(false);
		adTracker.complete();
		api.trigger('onAdCompleted',adsArr[0]);
		checkAvailableAds();
	}

	function onAdSlotStart () {
		if (adSlotArr.length === adsArr.length) {
			l.event('main', 'onAdSlotStart');
			api.trigger('onAdSlotStart', adSlotArr);
		}
	}

	function onAdSlotCompleted () {
		l.event('main', 'onAdSlotCompleted');
		api.trigger('onAdSlotCompleted', adSlotArr);

		if (adType == videoplaza.adrequest.AdRequester.InsertionPointType.ON_CONTENT_END) {
			resetPlayer();
		}
		else {
			startContent();
		}
	}

	function onAdSkipButtonClick () {
		l.event('main', 'onAdSkipButtonClick');
		player.removeEventListener('ended', onAdCompleted);
		player.removeEventListener('timeupdate', onAdTimeUpdate);
		enableClickThrough(false);
		topMenu.show(false);
		adTracker.close();
		api.trigger('onAdSkipButtonClick',adsArr[0]);
		checkAvailableAds();
	}

	function onAdTimeUpdate (evt) {
		adTracker.quartile(evt);
		topMenu.countdown(evt);
	}

	function onAdClickThrough () {
		l.event('main', 'onAdClickThrough');
		enableClickThrough(false);
		window.open(clickThroughUrl);
		adTracker.clickthrough();
		api.trigger('onAdClickThrough',adsArr[0]);
	}

	function onAdCompanionView () {
		l.event('main', 'onAdCompanionView');
		adTracker.creative_view(companion.getCompanion());
	}

	function onContentStart () {
		l.event('main', 'onContentStart');
		player.removeEventListener('canplaythrough', onContentStart);
		player.addEventListener('ended', onContentCompleted);
		adTracker.content_start();
		api.trigger('onContentStart');
	}

	function onContentCompleted () {
		l.event('main', 'onContentCompleted');
		player.removeEventListener('ended', onContentCompleted);
		api.trigger('onContentCompleted');
		displayPostroll();
	}

	doc.addEventListener(evt.AD_PLUGIN_INIT, onAdPluginInit);
	doc.addEventListener(evt.AD_REQUEST_SUCCESS, onAdRequestSuccess);
	doc.addEventListener(evt.AD_REQUEST_FAIL, onAdRequestFail);
	doc.addEventListener(evt.AD_REQUEST_THIRD_PARTY_SUCCESS, onAdRequestSuccess);
	doc.addEventListener(evt.AD_REQUEST_THIRD_PARTY_FAIL, onAdRequestFail);

	doc.dispatchEvent(new CustomEvent(evt.AD_PLUGIN_LOADED));
}());