utils.helpers = (function () {
	"use strict";
	return {
	    /**
		 * query URI for params foo=bar&baz=foo
		 * return Object {foo:'bar', baz:'foo'}
		 */
		queryURI : function (qs) {
			qs = decodeURIComponent(qs.split('+').join(' '));

			var params = {}, 
				tokens,
				re = /[?&]?([^=]+)=([^&]*)/g;

			while ((tokens = re.exec(qs)) !== null) {
				params[tokens[1]] = tokens[2];
			}
			return params;
		},
		objectToLowerCase : function (obj) {
			if (!obj) {
				return;
			}
			var key, keys = Object.keys(obj),
				n = keys.length,
				newobj = {};

			while (n--) {
			 	key = keys[n];
			 	newobj[key.toLowerCase()] = obj[key];
			}
			return newobj;
		},
		findSupportedVideo : function (player, mediaFiles) {
			for(var i=0; i < mediaFiles.length; i++) {
				if (player.canPlayType(mediaFiles[i].mimeType) === 'probably' ||
					player.canPlayType(mediaFiles[i].mimeType) === 'maybe') {
					return mediaFiles[i].url;
				}
			}
			return false;
		},
		isVideo : function (domElem) {
			try {
				return domElem.tagName.toLowerCase() === 'video';
			}
			catch(e) {
				return false;
			}
		},
		isIE : function () {
			try {
				return navigator.userAgent.search(/Trident|MSIE/g) !== -1;
			}
			catch(e) {
				return false;
			}
		},
		/**
		 * validate if obj has a property
		 * @param  {[type]} obj   [description]
		 * @param  {[type]} value [description]
		 */
		inObject : function (obj, property) {
			if (!obj) {
				return false;
			}
			else if (!obj.hasOwnProperty(property)) {
				return false;
			}
			return true;
		}
	};
}());
