view.topMenu = (function () {
	"use strict";

	var count = 0,
		evt = model.events,
		ui = model.ui,
		secUnit = ui.PLURAL_UNIT,
		player,
		topMenu,
		countdown,
		skipBtn,
		currentAd;


	function buildTopMenu (width, height) {
		var menuDiv = doc.createElement('div');
		menuDiv.id = 'topMenu';
		menuDiv.style.backgroundColor = 'rgba(000,0,0,0.6)';
        menuDiv.style.width = '100%';
        menuDiv.style.height = ui.TOP_MENU_HEIGHT + 'px';
        menuDiv.style.position = 'absolute';
        menuDiv.style.zIndex = '10005';
        return menuDiv;
	}

	function buildCountDown () {
		var countDiv = doc.createElement('div');
        countDiv.style.cssFloat = 'left';
        countDiv.style.width = (topMenu.width - (100 + 20)) + 'px';
        countDiv.style.height = ui.TOP_MENU_HEIGHT + 'px';
		countDiv.style.padding = '5px';
        countDiv.style.color = '#FFFFFF';
        countDiv.style.font = 'normal 14px Arial, Helvetica, sans-serif';
        return countDiv;
	}

	function buildSkipBtn () {
		var btn = doc.createElement('div');
		btn.style.cssFloat = 'right';
        btn.style.width = ui.TOP_MENU_HEIGHT + 'px';
        btn.style.height = ui.TOP_MENU_HEIGHT + 'px';
        btn.style.background = 'url('+ ui.CLOSE_BTN_IMAGE +') no-repeat right top ';
        btn.style.visibility = 'hidden';
		return btn;
	}

	function onSkipBtnClick () {
		doc.dispatchEvent(new CustomEvent(evt.AD_SKIP_BUTTON_CLICK));
	}
	
	/* ==========================================================================
    	Public
    ========================================================================== */
	return {
		build : function (playerElem, ad, width, height) {
			//only create once
			if (topMenu) {
				return;
			}

			currentAd = ad.creatives[0];
			player = playerElem;
			topMenu = buildTopMenu(width, height);
            countdown = buildCountDown();
            
            if (currentAd.skipButtonMode !== "never") {
            	skipBtn = buildSkipBtn();
            	topMenu.appendChild(skipBtn);
            }

            topMenu.appendChild(countdown);
            player.appendChild(topMenu);

            var css = document.createElement("style");
			css.type = "text/css";
			css.innerHTML = "*:fullscreen {width:100%;}";
			document.body.appendChild(css);
		},
		show : function (isVisible) {
			if (isVisible) {
				player.parentNode.insertBefore(topMenu, player);
				skipBtn.addEventListener(clickEvent, onSkipBtnClick);
			}
			else {
				topMenu.parentElement.removeChild(topMenu);
				skipBtn.removeEventListener(clickEvent, onSkipBtnClick);
				skipBtn.style.visibility = 'hidden';
			}
		},
		setSize : function (width, height) {
			if (width) {
				topMenu.style.width = width + 'px';
			}

			if (height) {
				topMenu.style.height = height + 'px';
			}
		},
		countdown : function (evt) {
			count = Math.floor(evt.target.duration - evt.target.currentTime) + 1;
	        secUnit = (count <= 1) ? ui.SINGULAR_UNIT : ui.PLURAL_UNIT;
	        countdown.innerHTML = ui.SPOT_COUNT_DOWN_END + ' ' + count + ' ' + secUnit;

	        //display close btn
	        if (Math.floor(evt.target.currentTime) === Math.floor(currentAd.skipOffset)) {
	            skipBtn.style.visibility = 'visible';
	        }
		}
	};
}());